#include "copyleft.h"

/*
    GEPASI - a simulator of metabolic pathways and other dynamical systems
    Copyright (C) 1989, 1992  Pedro Mendes
*/

/*************************************/
/*                                   */
/*         GWSIM - Simulation        */
/*        MS-WINDOWS front end       */
/*                                   */
/*          Simulation shell         */
/*             dialog box            */
/*                                   */
/*           QuickC/WIN 1.0          */
/*                                   */
/*   (include here compilers that    */
/*   compiled GWSIM successfully)    */
/*                                   */
/*************************************/


#include <windows.h>
#include <string.h>
#include <io.h>
#include "globals.h"
#include "defines.h"
#include "strtbl.h"
#include "gwsim.h"
#include "simgvar.h"
#include "basic.h"
#include "gep2.h"

#pragma alloc_text( CODE11, Execute, CheckSimEng )

#define HFILE_ERROR -1


/* calls the simulation engine to carry out the current simulation */

int Execute( HWND hWnd, HANDLE hInst )
{
 int hFile, nRc;
 char CommandLine[1024];
 char szString[256];
 char *ptr;


 lstrcpy( (LPSTR) CommandLine, (LPSTR) GepasiIni );
 ptr = strrchr( CommandLine, '\\' );
 *(ptr+1) = '\0';
 GetTempFileName( 0, (LPSTR) "GEP", 0, (LPSTR) TempFile );
 hFile = _lopen( (LPSTR) TempFile, OF_WRITE | OF_SHARE_COMPAT );
 nRc = _lwrite( hFile, FileName, lstrlen( FileName ) );
 nRc += _lwrite( hFile, (LPSTR) "\n", 1 );
 _lclose( hFile );
 if( nRc < ( lstrlen( (LPSTR) FileName ) +1 ) )
 {
  LoadString(hInst, IDS_ERR_NO_TEMP, szString, sizeof(szString));
  MessageBeep( MB_OK );
  MessageBox( hWnd, szString, FileTitle, MB_ICONINFORMATION );
  unlink( TempFile );
  return IDS_ERR_NO_TEMP;
 }
 if( options.debug )
 {
  lstrcat( (LPSTR) CommandLine, (LPSTR) "GEPASI_R.BAT " );
  GetModuleFileName( hInst, (LPSTR) szString, sizeof( szString ) );
  ptr = strrchr( szString, '\\' );
  *(ptr+1) = '\0';
  lstrcat( (LPSTR) CommandLine, (LPSTR) szString );
 }
 lstrcat( (LPSTR) CommandLine, (LPSTR) "GEPASI.EXE " );
 lstrcat( (LPSTR) CommandLine, (LPSTR) TempFile );
 lstrcat( (LPSTR) CommandLine, (LPSTR) " -d" );
 if( options.debug )
 {
  lstrcpy( (LPSTR) LogFile, (LPSTR) FileName );
  if ( ptr = strrchr( LogFile, '.' ) ) strcpy( ptr, ".LOG" );
  else strcat( ptr, ".LOG" );
  lstrcat( (LPSTR) CommandLine, (LPSTR) " " );
  lstrcat( (LPSTR) CommandLine, (LPSTR) LogFile );
 }
 nRc = WinExec( (LPSTR) CommandLine, SW_SHOWMINIMIZED );
 if( nRc<32 )
  ErrorHandler( hInst, nRc );
 else
 {
  if( ( TimerId = SetTimer( hWnd, 1, 750, fpCheckSimEng) ) == NULL )
  {
   MessageBeep( MB_OK );
   MessageBox( hWnd, "Unable to create timer.", (LPSTR) FileTitle, MB_ICONINFORMATION );
   return -1;
  }
  EnableMenuItem( hMenu, IDM_GO, MF_GRAYED );
  EnableWindow( hButton2, FALSE );
  DrawMenuBar( hWnd );
  running = 1;
 }

 return nRc;
}


/*
  Callback function that is called when timer goes off.
  Checks if the Simulation Engine is still running. If
  not plots the file and/or lists the report and/or the
  debug log
*/

void FAR PASCAL CheckSimEng(HWND hWnd, WORD msg, int idEvent, DWORD dwtime)
{
 OFSTRUCT OfStruct;
 int nRc;
 char szString[128];
 char CommandLine[192];
 char *ptr;

 /* if the temporary file still exists return				*/
 if( OpenFile( (LPSTR) TempFile, &OfStruct, OF_EXIST ) != HFILE_ERROR ) return;

 /* kill the timer											*/
 KillTimer( hWnd, TimerId );

 /* enable the Run selections								*/
 EnableMenuItem( hMenu, IDM_GO, MF_ENABLED );
 EnableWindow( hButton2, TRUE );
 DrawMenuBar( hWnd );
 InvalidateRect( hWnd, NULL, TRUE );

 running = 0;

 if( options.debug )
 {
  lstrcpy( (LPSTR) CommandLine, (LPSTR) "notepad " );
  lstrcat( (LPSTR) CommandLine, (LPSTR) LogFile );
  nRc = WinExec( (LPSTR) CommandLine, SW_SHOWNORMAL );
  if( nRc<32 ) ErrorHandler( hInst, nRc );
 }

 if( options.txt )
 {
  LoadString(hInst, IDS_REPORTQUERY, szString, sizeof(szString));
  MessageBeep( MB_ICONQUESTION );
  nRc = MessageBox(hWnd, (LPSTR) szString, (LPSTR) FileTitle, MB_ICONQUESTION | MB_YESNO );
  if( nRc == IDYES )
  {
   lstrcpy( (LPSTR) CommandLine, (LPSTR) "notepad " );
   lstrcat( (LPSTR) CommandLine, (LPSTR) FileName );
   if ( ptr = strrchr( CommandLine, '.' ) ) strcpy( ptr, ".TXT" );
   else strcat( ptr, ".TXT" );
   nRc = WinExec( (LPSTR) CommandLine, SW_SHOWNORMAL );
   if( nRc<32 ) ErrorHandler( hInst, nRc );
  }
 }
}
